package cc.redberry.pipe.blocks;

import cc.redberry.pipe.OutputPortUninterruptible;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * Implementation of {@link cc.redberry.pipe.OutputPort} interface with proper synchronization for close() method.
 *
 * @param <T> type of stream object
 */
public abstract class AbstractOutputPortUninterruptible<T> implements OutputPortUninterruptible<T> {
    protected final ReadWriteLock readWriteLock = new ReentrantReadWriteLock();
    protected boolean closed = false;

    @Override
    public final T take() {
        final Lock lock = readWriteLock.readLock();
        lock.lock();
        try {
            if (closed)
                return null;
            T value = _take();

            //Not synchronized...
            if (value == null)
                closed = true;

            return value;
        } finally {
            lock.unlock();
        }
    }

    public abstract T _take();

    /**
     * (!) Do not run this method from inside {@link #_take()} method.
     */
    @Override
    public final void close() {
        final Lock lock = readWriteLock.writeLock();
        lock.lock();
        try {
            closed = true;
            _close();
        } finally {
            lock.unlock();
        }
    }

    public void _close() {
    }

    public final boolean isClosed() {
        final Lock lock = readWriteLock.readLock();
        lock.lock();
        try {
            return closed;
        } finally {
            lock.unlock();
        }
    }
}
